.. include:: general/positional-arguments.rst.inc

.. include:: general/repository-urls.rst.inc

.. include:: general/repository-locations.rst.inc

.. include:: general/logging.rst.inc

.. include:: general/return-codes.rst.inc

.. _env_vars:

.. include:: general/environment.rst.inc

.. _file-systems:

.. include:: general/file-systems.rst.inc

.. include:: general/units.rst.inc

.. include:: general/date-time.rst.inc

.. include:: general/resources.rst.inc

.. _platforms:

.. include:: general/file-metadata.rst.inc
